program RemotePrimeServer;
{$APPTYPE CONSOLE}

(*
 * To compile, make sure you have the RemotePrimeCalc.dll in this same directory, 
 * generated from the RemotePrimeCalc.cs (which is located in the ..\c#\assemblies
 * directory).
 *
 * dccil -luRemotePrimeCalc RemotePrimeServer.dpr 
 *
 * Use the RemotePrimeClient application to connect to this server app.
 *
 *)

uses
  System.Drawing,
  System.Collections,
  System.ComponentModel,
  System.Windows.Forms,
  System.Runtime.Remoting,
  System.Runtime.Remoting.Channels.Http,
  System.Runtime.Remoting.Channels,
  System.Runtime.Remoting.Lifetime,
  System.Threading,
  Borland.VCL.Classes,
  RickRoss.BorCon2003.RemotePrimeCalc; // imported from RemotePrimeServer.dll

const
  SERVER_URI  = 'BorCon2003PrimeCalcServer.soap';
  PORT_NUMBER = 2013;

type 
  TClientNameDelegate  = procedure ( clientName : string ) of object;
  TPrimeNumberDelegate = procedure ( number : string ) of object;

  TPrimeCalcServer = class;

  TWinForm = class(System.Windows.Forms.Form)
  private
    components     : Container;
    FControl       : Control;
    lbClients      : ListBox;
    Label1         : System.Windows.Forms.Label;
    mmPrimeNumbers : TextBox;
    Label2         : System.Windows.Forms.Label;
    btnStop        : Button;
  
    Fpcs           : TPrimeCalcServer;

    procedure InitializeComponent; 
    procedure RegisterServerObject;
    procedure btnStop_OnClick(sender : System.Object; e : System.EventArgs );
  strict protected
    procedure Dispose( disposing : boolean ); override;
  public
    constructor Create; 
    procedure AddPrimeNumber( number : string ); 
    procedure AddClient( clientName : string ); 

    // any control is fine. We only need it to guarantee that any
    // methods we call will be called on the main thread.
    property AControl  : Control read FControl; 
  end;

  TPrimeCalcServer = class( MarshalByRefObject, IPrimeNumberServer , ISponsor )
  private
    FRangeLock  : Mutex;
    FWinForm    : TWinForm;
    FClientList : TStrings;
    FNumClients : integer;
    FLastRange  : Int64;
    FStopNow    : boolean;

    procedure GetNextRange( var start, stop : Int64 );
  public
    constructor Create( aWinForm : TWinForm );
    destructor  Destroy; override;
    function    InitializeLifetimeService : System.Object; override;
    function    Renewal( lease : ILease ) : TimeSpan;
    function    ReadyForWork( clientName : string ) : PrimeNumberData;
    procedure   PrimeNumbers( data : PrimeNumberData );
    property    StopNow : boolean read FStopNow write FStopNow;
  end;


{ TWinForm }

constructor TWinForm.Create;
begin
  inherited;
  InitializeComponent;
  FControl := Control( self.lbClients );
  RegisterServerObject;
end;

procedure TWinForm.InitializeComponent;
begin
  self.lbClients      := System.Windows.Forms.ListBox.Create;
  self.Label1         := System.Windows.Forms.Label.Create;
  self.mmPrimeNumbers := System.Windows.Forms.TextBox.Create;
  self.Label2         := System.Windows.Forms.Label.Create;
  self.btnStop        := System.Windows.Forms.Button.Create; 
  self.SuspendLayout();

  // lbclients
  //
  self.lbClients.Location   := System.Drawing.Point.Create( 16, 32);
  self.lbClients.Name       := 'lbClients';
  self.lbClients.Size       := System.Drawing.Size.Create( 320, 355);
  self.lbClients.TabIndex   := 1;
  self.lbClients.Name       := 'lbclients';

  // Label1
  //
  self.Label1.Location      := System.Drawing.Point.Create( 16, 16);
  self.Label1.Name          := 'Label1';
  self.Label1.Size          := System.Drawing.Size.Create( 129, 13);
  self.Label1.TabIndex      := 3;
  self.Label1.Text          := 'Clients Connected / Range';
  
  // Label2
  //
  self.Label2.Location      := System.Drawing.Point.Create(392, 16);
  self.Label2.Name          := 'Label2';
  self.Label2.TabIndex      := 3;
  self.Label2.Text          := 'Prime Numbers';

  // Prime Numbers Memo
  // 
  self.mmPrimeNumbers.Location   := System.Drawing.Point.Create( 392, 32 );
  self.mmPrimeNumbers.Multiline  := true;
  self.mmPrimeNumbers.Name       := 'mmPrimeNumbers';
  self.mmPrimeNumbers.Size       := System.Drawing.Size.Create( 312, 416 );
  self.mmPrimeNumbers.TabIndex   := 4;
  self.mmPrimeNumbers.ScrollBars := ScrollBars.Vertical;

  self.btnStop.Location     := System.Drawing.Point.Create( 127, 395 );
  self.btnStop.Name         := 'btnStop';
  self.btnStop.TabIndex     := 5;
  self.btnStop.Text         := 'Stop';
  self.btnStop.add_Click( self.btnStop_OnClick );
  
  // WinForm
  //
  self.Text := 'Prime Number Server';
  self.AutoScaleBaseSize := System.Drawing.Size.Create(5, 13);
  self.ClientSize        := System.Drawing.Size.Create(728, 477);
  self.Controls.Add(self.lbClients);
  self.Controls.Add(self.Label1);
  self.Controls.Add(self.mmPrimeNumbers);
  self.Controls.Add(self.Label2);
  self.Controls.Add(self.btnStop);
  self.Name          := 'WinForm';
  self.StartPosition := System.Windows.Forms.FormStartPosition.CenterScreen;
  self.ResumeLayout(false);
end;

procedure TWinForm.RegisterServerObject;
var
  aChannel : HttpChannel;
begin
  writeln('Registering server object.');

  writeln('Server: Creating channel');
  aChannel := HttpChannel.Create(2003);
  
  writeln('Server: Registering channel...');
  ChannelServices.RegisterChannel( aChannel );

  writeln('Server: Creating prime calc server...');
  Fpcs := TPrimeCalcServer.Create(self);

  writeln('Server: Registering type...');
  RemotingServices.Marshal(Fpcs, 'BorCon2003PrimeCalcServer.soap');
end;

procedure TWinForm.btnStop_OnClick(sender : System.Object; e : System.EventArgs );
begin
  Fpcs.StopNow := true;
end;

procedure TWinForm.Dispose( disposing : boolean ); 
begin
  if (disposing) then
  begin
    if (components <> nil) then
    begin
      components.Dispose();
    end;
  end;
  inherited;
end;

procedure TWinForm.AddPrimeNumber( number : string ); 
var
  len : integer;
  tmpArray : array of string;

begin
  Application.DoEvents();
  len := Length(self.mmPrimeNumbers.Lines);
  tmpArray := self.mmPrimeNumbers.Lines;
  SetLength(tmpArray, len+1);
  tmpArray[len] := number;
  self.mmPrimeNumbers.Lines := tmpArray; 
end;

procedure TWinForm.AddClient( clientName : string ); 
begin
  lbClients.items.Add( clientName );
end;

{ TPrimeCalcServer }

constructor TPrimeCalcServer.Create( aWinForm : TWinForm);
const
  MIN_CLIENTS = 5;

begin
  inherited Create;
  FWinForm := aWinForm;
  FClientList := TStringList.Create;
  FRangeLock  := Mutex.Create;
  FNumClients := 0;
end;

destructor  TPrimeCalcServer.Destroy;
begin
  FClientList.Free;
  FRangeLock.Free;
  inherited;
end;

function  TPrimeCalcServer.InitializeLifetimeService : System.Object; 
var
  lease : Ilease;
begin
  lease := inherited InitializeLifetimeService() as ILease;
  if (lease.CurrentState = LeaseState.Initial) then
  begin
    lease.InitialLeaseTime := TimeSpan.FromMinutes(1);
    lease.Register( self );
  end;
  Result := lease;
end;

function TPrimeCalcServer.Renewal(lease : ILease ) : TimeSpan;
begin
  // renew this lease for another 5 minutes...
  lease.Renew( TimeSpan.FromMinutes( 5 ) );
end;

procedure TPrimeCalcServer.GetNextRange( var start, stop : Int64 );
const
  //RANGE_SIZE  = 1000000;
  RANGE_SIZE = 10000;

begin
  if not FRangeLock.WaitOne() then
    raise Exception.Create('Error waiting for range lock mutex');
  try
    start := FLastRange+1;
    stop  := start + RANGE_SIZE;
    FLastRange := stop
  finally
    FRangeLock.ReleaseMutex();
  end;
end;

function TPrimeCalcServer.ReadyForWork( clientName : string ) : PrimeNumberData;
var
  start, stop : Int64;
  parms       : array of System.Object;
  lbDelegate  : TClientNameDelegate;

begin
  // add client to array
  Result := PrimeNumberData.Create;
  Result.TimeToStop := FStopNow;
  if not FStopNow then
  begin
    // does this client already exist?
    if (FClientList.IndexOf( clientName ) < 0) then
    begin
      FClientList.Add( clientName );

      // add client to listbox
      SetLength( parms, 1 );
      parms[0]   := System.Object(clientName);
      lbDelegate := @FwinForm.AddClient;
      FWinForm.AControl.Invoke( System.Delegate(@lbDelegate), parms );
    end;
  
    // dispatch next set of prime numbers 
    GetNextRange( start, stop );

    Result.ClientName := clientName;
    Result.StartAt    := start;
    Result.StopAt     := stop;
  end;  
end;

procedure TPrimeCalcServer.PrimeNumbers( data : PrimeNumberData );
var
  i          : integer;
  parms      : array of System.Object;
  tbDelegate : TPrimeNumberDelegate; 

begin
  SetLength( parms, 1 );
  tbDelegate := @FWinForm.AddPrimeNumber;
  for i:=0 to length(data.PrimeNumbers)-1 do
  begin
    parms[0] := System.Object( data.PrimeNumbers[i] ).ToString();
    FWinForm.AControl.Invoke( System.Delegate(@tbDelegate), parms );
  end;  
end;

var
  frm : TWinForm;
    
begin
  frm := TWinForm.Create;
  Application.Run(frm); 
end.