unit RickRoss.RemotePrimeCalc;

(*
 * RemotePrimeCalc defines the interface between the server and client.
 * It contains a serializable PrimeNumber class, an array of prime numbers
 * and the IPrimeNumberServer interface. This unit is included in the 
 * RemotePrimeCalc.dpk and the resulting DLL must be distributed with the 
 * client and the server.
 *
 * Written by Rick Ross (http://www.rick-ross.com)
 *
 *)

interface

type
  TPrimeNumberArray = array of Int64;

  [Serializable()]
  PrimeNumberData = class(System.Object)
  strict private
    _ClientName   : string;
    _StartAt      : Int64;
    _StopAt       : Int64;
    _PrimeNumbers : TPrimeNumberArray;
    _TimeToStop   : boolean;
  public
    property ClientName   : string read _ClientName write _ClientName;
    property StartAt      : Int64  read _StartAt    write _StartAt;
    property StopAt       : Int64  read _StopAt     write _StopAt;
    property PrimeNumbers : TPrimeNumberArray read _PrimeNumbers write _PrimeNumbers;
    property TimeToStop   : boolean read _TimeToStop write _TimeToStop;
  end;

  IPrimeNumberServer = interface
    // client says "I need some work to do"
    // server says "Okay... here is what you need to do"
    function ReadyForWork( clientName : string ) : PrimeNumberData;

    // client says "Here is the answer"
    // server accepts the data
    procedure PrimeNumbers( results : PrimeNumberData);    
  end;

implementation

end.