unit WinForm;

(*
 * WinForm.pas
 *   This unit contains the user interface for the RemotePrimeNumberServer.
 *   It requires the RemotePrimeCalc.dll and the RemotePrimeCalc.dcpil files
 *   in order for it to be compiled. See the buildall.bat file for complete
 *   build instructions.
 *
 * Written by Rick Ross (http://www.rick-ross.com)
 *
 *)

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.Data, System.IO,
  System.Runtime.Remoting,
  System.Runtime.Remoting.Channels.Http,
  System.Runtime.Remoting.Channels,
  System.Runtime.Remoting.Lifetime,
  System.Threading,
  Borland.VCL.Classes,
  RickRoss.RemotePrimeCalc; // imported from RemotePrimeServer.dll

const
  SERVER_URI  = 'PrimeCalcServer.soap';
  PORT_NUMBER = 2003;

type
  TClientNameDelegate  = procedure ( clientName : string ) of object;
  TPrimeNumberDelegate = procedure ( number : string ) of object;
  TUpdateNumberDelegate = procedure ( number : Int64 ) of object;

  TPrimeCalcServer = class;

  TWinForm = class(System.Windows.Forms.Form)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    Panel1: System.Windows.Forms.Panel;
    Label1: System.Windows.Forms.Label;
    Panel2: System.Windows.Forms.Panel;
    Panel3: System.Windows.Forms.Panel;
    Label2: System.Windows.Forms.Label;
    Label3: System.Windows.Forms.Label;
    lbClients: System.Windows.Forms.ListBox;
    Button1: System.Windows.Forms.Button;
    mmPrimeNumbers: System.Windows.Forms.TextBox;
    btnSavePrimeNumbers: System.Windows.Forms.Button;
    dlgSaveFile: System.Windows.Forms.SaveFileDialog;
    Label5: System.Windows.Forms.Label;
    tbHighestNumber: System.Windows.Forms.TextBox;
    Label4: System.Windows.Forms.Label;
    tbNumPrimes: System.Windows.Forms.TextBox;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
    procedure btnSavePrimeNumbers_Click(sender: System.Object; e: System.EventArgs);
    procedure Button1_Click(sender: System.Object; e: System.EventArgs);
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    FControl       : Control;
    Fpcs           : TPrimeCalcServer;
    FCurNumPrimes  : Int64;
    procedure Dispose(Disposing: Boolean); override;
    procedure RegisterServerObject;
  public
    constructor Create;
    procedure AddPrimeNumber( number : string );
    procedure AddClient( clientName : string );
    procedure UpdateLatestNumber( number : Int64 );
    procedure UpdateNumberOfPrimes( number : Int64 );

    // any control is fine. We only need it to guarantee that any
    // methods we call will be called on the main thread.
    property AControl  : Control read FControl;
  end;

  // This is the class that is responsible for implementing the IPrimeNumberServer
  // interface, where clients will connect and request a unit of work
  // (e.g. a range of numbers in which to look for prime numbers)
  TPrimeCalcServer = class( MarshalByRefObject, IPrimeNumberServer , ISponsor )
  private
    FRangeLock  : Mutex;
    FWinForm    : TWinForm;
    FClientList : TStrings;
    FNumClients : integer;
    FLastRange  : Int64;
    FStopNow    : boolean;

    procedure GetNextRange( var start, stop : Int64 );
  public
    constructor Create( aWinForm : TWinForm );
    destructor  Destroy; override;
    function    InitializeLifetimeService : System.Object; override;
    function    Renewal( lease : ILease ) : TimeSpan;
    function    ReadyForWork( clientName : string ) : PrimeNumberData;
    procedure   PrimeNumbers( data : PrimeNumberData );
    property    StopNow : boolean read FStopNow write FStopNow;
  end;

implementation

{$R 'WinForm.TWinForm.resources'}

uses
  System.Globalization;

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TWinForm.InitializeComponent;
begin
  Self.Panel1 := System.Windows.Forms.Panel.Create;
  Self.Label1 := System.Windows.Forms.Label.Create;
  Self.Panel2 := System.Windows.Forms.Panel.Create;
  Self.Button1 := System.Windows.Forms.Button.Create;
  Self.lbClients := System.Windows.Forms.ListBox.Create;
  Self.Label2 := System.Windows.Forms.Label.Create;
  Self.Panel3 := System.Windows.Forms.Panel.Create;
  Self.tbNumPrimes := System.Windows.Forms.TextBox.Create;
  Self.Label4 := System.Windows.Forms.Label.Create;
  Self.tbHighestNumber := System.Windows.Forms.TextBox.Create;
  Self.Label5 := System.Windows.Forms.Label.Create;
  Self.btnSavePrimeNumbers := System.Windows.Forms.Button.Create;
  Self.mmPrimeNumbers := System.Windows.Forms.TextBox.Create;
  Self.Label3 := System.Windows.Forms.Label.Create;
  Self.dlgSaveFile := System.Windows.Forms.SaveFileDialog.Create;
  Self.Panel1.SuspendLayout;
  Self.Panel2.SuspendLayout;
  Self.Panel3.SuspendLayout;
  Self.SuspendLayout;
  // 
  // Panel1
  // 
  Self.Panel1.Controls.Add(Self.Label1);
  Self.Panel1.Dock := System.Windows.Forms.DockStyle.Top;
  Self.Panel1.Location := System.Drawing.Point.Create(0, 0);
  Self.Panel1.Name := 'Panel1';
  Self.Panel1.Size := System.Drawing.Size.Create(544, 56);
  Self.Panel1.TabIndex := 0;
  // 
  // Label1
  // 
  Self.Label1.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 12, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.Label1.Location := System.Drawing.Point.Create(144, 16);
  Self.Label1.Name := 'Label1';
  Self.Label1.Size := System.Drawing.Size.Create(256, 24);
  Self.Label1.TabIndex := 0;
  Self.Label1.Text := 'Remote Prime Number Server';
  // 
  // Panel2
  // 
  Self.Panel2.Controls.Add(Self.Button1);
  Self.Panel2.Controls.Add(Self.lbClients);
  Self.Panel2.Controls.Add(Self.Label2);
  Self.Panel2.Dock := System.Windows.Forms.DockStyle.Left;
  Self.Panel2.Location := System.Drawing.Point.Create(0, 56);
  Self.Panel2.Name := 'Panel2';
  Self.Panel2.Size := System.Drawing.Size.Create(208, 429);
  Self.Panel2.TabIndex := 1;
  // 
  // Button1
  // 
  Self.Button1.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 8.25, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.Button1.Location := System.Drawing.Point.Create(56, 392);
  Self.Button1.Name := 'Button1';
  Self.Button1.Size := System.Drawing.Size.Create(96, 23);
  Self.Button1.TabIndex := 2;
  Self.Button1.Text := 'Stop Clients';
  Include(Self.Button1.Click, Self.Button1_Click);
  // 
  // lbClients
  // 
  Self.lbClients.Location := System.Drawing.Point.Create(16, 32);
  Self.lbClients.Name := 'lbClients';
  Self.lbClients.Size := System.Drawing.Size.Create(176, 342);
  Self.lbClients.TabIndex := 1;
  // 
  // Label2
  // 
  Self.Label2.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 8.25, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.Label2.Location := System.Drawing.Point.Create(24, 8);
  Self.Label2.Name := 'Label2';
  Self.Label2.Size := System.Drawing.Size.Create(152, 16);
  Self.Label2.TabIndex := 0;
  Self.Label2.Text := 'Clients Connected / Ran';
  // 
  // Panel3
  // 
  Self.Panel3.Controls.Add(Self.tbNumPrimes);
  Self.Panel3.Controls.Add(Self.Label4);
  Self.Panel3.Controls.Add(Self.tbHighestNumber);
  Self.Panel3.Controls.Add(Self.Label5);
  Self.Panel3.Controls.Add(Self.btnSavePrimeNumbers);
  Self.Panel3.Controls.Add(Self.mmPrimeNumbers);
  Self.Panel3.Controls.Add(Self.Label3);
  Self.Panel3.Dock := System.Windows.Forms.DockStyle.Fill;
  Self.Panel3.Location := System.Drawing.Point.Create(208, 56);
  Self.Panel3.Name := 'Panel3';
  Self.Panel3.Size := System.Drawing.Size.Create(336, 429);
  Self.Panel3.TabIndex := 2;
  // 
  // tbNumPrimes
  // 
  Self.tbNumPrimes.Location := System.Drawing.Point.Create(168, 40);
  Self.tbNumPrimes.Name := 'tbNumPrimes';
  Self.tbNumPrimes.Size := System.Drawing.Size.Create(152, 20);
  Self.tbNumPrimes.TabIndex := 8;
  Self.tbNumPrimes.Text := '0';
  Self.tbNumPrimes.TextAlign := System.Windows.Forms.HorizontalAlignment.Right;
  // 
  // Label4
  // 
  Self.Label4.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 8.25, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.Label4.Location := System.Drawing.Point.Create(16, 42);
  Self.Label4.Name := 'Label4';
  Self.Label4.Size := System.Drawing.Size.Create(152, 16);
  Self.Label4.TabIndex := 7;
  Self.Label4.Text := 'Number of Primes found';
  // 
  // tbHighestNumber
  // 
  Self.tbHighestNumber.Location := System.Drawing.Point.Create(168, 8);
  Self.tbHighestNumber.Name := 'tbHighestNumber';
  Self.tbHighestNumber.Size := System.Drawing.Size.Create(152, 20);
  Self.tbHighestNumber.TabIndex := 6;
  Self.tbHighestNumber.Text := '0';
  Self.tbHighestNumber.TextAlign := System.Windows.Forms.HorizontalAlignment.Right;
  // 
  // Label5
  // 
  Self.Label5.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 8.25, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.Label5.Location := System.Drawing.Point.Create(16, 10);
  Self.Label5.Name := 'Label5';
  Self.Label5.Size := System.Drawing.Size.Create(120, 16);
  Self.Label5.TabIndex := 5;
  Self.Label5.Text := 'Searching from 1 to';
  // 
  // btnSavePrimeNumbers
  // 
  Self.btnSavePrimeNumbers.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 8.25, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.btnSavePrimeNumbers.Location := System.Drawing.Point.Create(72, 392);
  Self.btnSavePrimeNumbers.Name := 'btnSavePrimeNumbers';
  Self.btnSavePrimeNumbers.Size := System.Drawing.Size.Create(192, 23);
  Self.btnSavePrimeNumbers.TabIndex := 2;
  Self.btnSavePrimeNumbers.Text := 'Save Prime Numbers to a file';
  Include(Self.btnSavePrimeNumbers.Click, Self.btnSavePrimeNumbers_Click);
  // 
  // mmPrimeNumbers
  // 
  Self.mmPrimeNumbers.Location := System.Drawing.Point.Create(16, 104);
  Self.mmPrimeNumbers.Multiline := True;
  Self.mmPrimeNumbers.Name := 'mmPrimeNumbers';
  Self.mmPrimeNumbers.ScrollBars := System.Windows.Forms.ScrollBars.Vertical;
  Self.mmPrimeNumbers.Size := System.Drawing.Size.Create(304, 272);
  Self.mmPrimeNumbers.TabIndex := 1;
  Self.mmPrimeNumbers.Text := '';
  // 
  // Label3
  // 
  Self.Label3.Font := System.Drawing.Font.Create('Microsoft Sans Serif', 8.25, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, (System.Byte(0)));
  Self.Label3.Location := System.Drawing.Point.Create(88, 80);
  Self.Label3.Name := 'Label3';
  Self.Label3.Size := System.Drawing.Size.Create(160, 16);
  Self.Label3.TabIndex := 0;
  Self.Label3.Text := 'List of Prime Numbers Found';
  // 
  // TWinForm
  // 
  Self.AutoScaleBaseSize := System.Drawing.Size.Create(5, 13);
  Self.ClientSize := System.Drawing.Size.Create(544, 485);
  Self.Controls.Add(Self.Panel3);
  Self.Controls.Add(Self.Panel2);
  Self.Controls.Add(Self.Panel1);
  Self.Name := 'TWinForm';
  Self.Text := 'Remote Prime Number Server';
  Self.Panel1.ResumeLayout(False);
  Self.Panel2.ResumeLayout(False);
  Self.Panel3.ResumeLayout(False);
  Self.ResumeLayout(False);
end;
{$ENDREGION}

procedure TWinForm.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

constructor TWinForm.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
  FControl := Control( self.lbClients );
  // Make our IPrimeNumberServer object available to remoting clients
  RegisterServerObject;
end;

procedure TWinForm.RegisterServerObject;
var
  aChannel : HttpChannel;
begin
  writeln('Registering server object.');

  writeln('Server: Creating channel on port ', PORT_NUMBER);
  aChannel := HttpChannel.Create(PORT_NUMBER);

  writeln('Server: Registering channel...');
  ChannelServices.RegisterChannel( aChannel );

  writeln('Server: Creating prime calc server...');
  Fpcs := TPrimeCalcServer.Create(self);

  writeln('Server: Registering type... on uri ', SERVER_URI);
  RemotingServices.Marshal(Fpcs, SERVER_URI);

  writeln('Server is available at: http://<computer name>:',PORT_NUMBER,'/',SERVER_URI);
end;

// Save the list of Prime numbers in a file
//
procedure TWinForm.btnSavePrimeNumbers_Click(sender: System.Object; e: System.EventArgs);
var
  sw : StreamWriter;
  i  : integer;

begin
  dlgSaveFile.Title := 'File to save prime numbers in';
  dlgSaveFile.OverwritePrompt := true;
  if (dlgSaveFile.ShowDialog = System.Windows.Forms.DialogResult.OK) then
  begin
    sw := StreamWriter.Create( dlgSaveFile.FileName );
    try
      for i:=0 to Length(mmPrimeNumbers.Lines)-1 do
      begin
        sw.WriteLine( mmPrimeNumbers.Lines[i] );
      end;
    finally
      sw.Close;
    end;
  end;
end;

//
// updates the list of prime numbers with the given number
// Note: This method *must* be called from the UI thread and not from
//       a worker thread.
//
procedure TWinForm.AddPrimeNumber( number : string );
var
  len : integer;
  tmpArray : array of string;

begin
  Application.DoEvents();
  len := Length(self.mmPrimeNumbers.Lines);
  tmpArray := self.mmPrimeNumbers.Lines;
  SetLength(tmpArray, len+1);
  tmpArray[len] := number;
  self.mmPrimeNumbers.Lines := tmpArray;
end;

// Adds a client to our listbox
// Note: This method *must* be called from the UI thread and not from
//       a worker thread.

procedure TWinForm.AddClient( clientName : string );
begin
  lbClients.items.Add( clientName );
end;

// Updates the greatest number that is currently being searched.
// Note: This method *must* be called from the UI thread and not from
//       a worker thread.

procedure TWinForm.UpdateLatestNumber( number : Int64 );
begin
  tbHighestNumber.Text := IntToStr( number );
end;

// Keeps track of the number of prime numbers we've found
// Note: This method *must* be called from the UI thread and not from
//       a worker thread.

procedure TWinForm.UpdateNumberOfPrimes( number : Int64 );
begin
  inc(FCurNumPrimes, number);
  tbNumPrimes.Text := IntToStr( FCurNumPrimes );
end;

// Sets a boolean indicator to inform the clients to stop working so hard

procedure TWinForm.Button1_Click(sender: System.Object; e: System.EventArgs);
begin
  Fpcs.StopNow := true;
end;

{ TPrimeCalcServer }

constructor TPrimeCalcServer.Create( aWinForm : TWinForm);
const
  MIN_CLIENTS = 5;

begin
  inherited Create;
  FWinForm    := aWinForm;
  FClientList := TStringList.Create;
  FRangeLock  := Mutex.Create;
  FNumClients := 0;
end;

destructor  TPrimeCalcServer.Destroy;
begin
  FClientList.Free;
  FRangeLock.Free;
  inherited;
end;

// In order to make sure that our server is not garbage collected, we
// have implemented the ISponsor interface. This allows us to decide when
// our server can be garbage collected. In this case, we register ourselves
// as the lease sponsor.

function  TPrimeCalcServer.InitializeLifetimeService : System.Object;
var
  lease : Ilease;
begin
  lease := inherited InitializeLifetimeService() as ILease;
  if (lease.CurrentState = LeaseState.Initial) then
  begin
    lease.InitialLeaseTime := TimeSpan.FromMinutes(1);
    lease.Register( self );
  end;
  Result := lease;
end;

//  The remoting framework periodically calls Renewal to see if we want to
//  renew the lease on our server. In this case, we set the time to 5 minutes

function TPrimeCalcServer.Renewal(lease : ILease ) : TimeSpan;
begin
  // renew this lease for another 5 minutes...
  lease.Renew( TimeSpan.FromMinutes( 5 ) );
end;

//
// GetNextRange uses a mutex to serialize access to this shared method.
// Multiple clients may be attempting to get the next range and it is
// pointless to have more than one client working on the same range of numbers

procedure TPrimeCalcServer.GetNextRange( var start, stop : Int64 );
const
  RANGE_SIZE = 10000;

begin
  if not FRangeLock.WaitOne() then
    raise Exception.Create('Error waiting for range lock mutex');
  try
    start := FLastRange+1;
    stop  := start + RANGE_SIZE;
    FLastRange := stop
  finally
    FRangeLock.ReleaseMutex();
  end;
end;

// ReadyForWork is called repeatedly by clients when they are ready for a unit
// of work (e.g. the next range of numbers to search for prime numbers)
// Since this method is executed on a thread that is not the UI thread, it
// uses a Control's Invoke method, to perform any GUI updating.
//

function TPrimeCalcServer.ReadyForWork( clientName : string ) : PrimeNumberData;
var
  start, stop : Int64;
  parms       : array of System.Object;
  lbDelegate  : TClientNameDelegate;
  numDelegate : TUpdateNumberDelegate;

begin
  writeln('Client ',clientName,' is ready for work!');
  // add client to array
  Result := PrimeNumberData.Create;
  Result.TimeToStop := FStopNow;
  if not FStopNow then
  begin
    SetLength( parms, 1 );
    // does this client already exist?
    if (FClientList.IndexOf( clientName ) < 0) then
    begin
      FClientList.Add( clientName );

      // add client to listbox
      parms[0]   := System.Object(clientName);
      lbDelegate := @FwinForm.AddClient;
      FWinForm.AControl.Invoke( System.Delegate(@lbDelegate), parms );
    end;

    // dispatch next set of prime numbers
    GetNextRange( start, stop );
    parms[0] := System.Object(stop);
    numDelegate := @FwinForm.UpdateLatestNumber;
    FWinForm.AControl.Invoke( System.Delegate(@numDelegate), parms );

    Result.ClientName := clientName;
    Result.StartAt    := start;
    Result.StopAt     := stop;
  end;
end;

//
// PrimeNumbers is called by the client when it has a set of prime numbers in
// a given range. Since this method is updating the UI, it uses a control's
// Invoke method to udpate the User Interface safely.

procedure TPrimeCalcServer.PrimeNumbers( data : PrimeNumberData );
var
  i          : integer;
  parms      : array of System.Object;
  tbDelegate : TPrimeNumberDelegate;
  numDelegate : TUpdateNumberDelegate;

begin
  SetLength( parms, 1 );
  tbDelegate := @FWinForm.AddPrimeNumber;
  numDelegate := @FWinForm.UpdateNumberOfPrimes;
  for i:=0 to length(data.PrimeNumbers)-1 do
  begin
    parms[0] := System.Object( data.PrimeNumbers[i] ).ToString();
    FWinForm.AControl.Invoke( System.Delegate(@tbDelegate), parms );
  end;

  parms[0] := System.Object( Length( data.PrimeNumbers ) );
  FWinForm.AControl.Invoke( System.Delegate(@numDelegate), parms );
end;

end.
